@@ -38,13 +38,22 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
                              'http://www.sat.gob.mx/Pagos20 ' +
                              'http://www.sat.gob.mx/sitio_internet/cfd/Pagos/Pagos20.xsd')
         elif self.complemento == COMPLEMENTO_CARTA_PORTE:
+            cartaporte_data = self.data.get('complemento_cartaporte', {})
             dom.setAttribute('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance')
-            dom.setAttribute('xmlns:cartaporte20', 'http://www.sat.gob.mx/CartaPorte20')
-            dom.setAttribute('xsi:schemaLocation',
-                             'http://www.sat.gob.mx/cfd/4 ' +
-                             'http://www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd ' +
-                             'http://www.sat.gob.mx/CartaPorte20 ' +
-                             'http://www.sat.gob.mx/sitio_internet/cfd/CartaPorte/CartaPorte20.xsd')
+            if cartaporte_data.get('Version') == '3.0':
+                dom.setAttribute('xmlns:cartaporte30', 'http://www.sat.gob.mx/CartaPorte30')
+                dom.setAttribute('xsi:schemaLocation',
+                                 'http://www.sat.gob.mx/cfd/4 ' +
+                                 'http://www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd ' +
+                                 'http://www.sat.gob.mx/CartaPorte30 ' +
+                                 'http://www.sat.gob.mx/sitio_internet/cfd/CartaPorte/CartaPorte30.xsd')
+            else:
+                dom.setAttribute('xmlns:cartaporte20', 'http://www.sat.gob.mx/CartaPorte20')
+                dom.setAttribute('xsi:schemaLocation',
+                                 'http://www.sat.gob.mx/cfd/4 ' +
+                                 'http://www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd ' +
+                                 'http://www.sat.gob.mx/CartaPorte20 ' +
+                                 'http://www.sat.gob.mx/sitio_internet/cfd/CartaPorte/CartaPorte20.xsd')
         else:
             dom.setAttribute('xsi:schemaLocation', 'http://www.sat.gob.mx/cfd/4 http://www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd')
             dom.setAttribute('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance')
@@ -154,7 +163,7 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
         try:
             emisor = self.dom.createElement('cfdi:Emisor')
             emisor.setAttribute('Rfc', self.data.get('emisor_rfc'))
-            emisor.setAttribute('Nombre', self.data.get('emisor_nombre', '').strip())
+            emisor.setAttribute('Nombre', replace(self.data.get('emisor_nombre', '')).strip())
             emisor.setAttribute('RegimenFiscal', self.data.get('regimen_fiscal'))
             return emisor
         except Exception as e:
@@ -164,7 +173,7 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
         try:
             receptor = self.dom.createElement('cfdi:Receptor')
             receptor.setAttribute('Rfc', self.data.get('receptor_rfc'))
-            receptor.setAttribute('Nombre',self.data.get('receptor_nombre', '').strip())
+            receptor.setAttribute('Nombre', replace(self.data.get('receptor_nombre', '')).strip())
             receptor.setAttribute('DomicilioFiscalReceptor', self.data.get('domicilio_fiscal_receptor', ''))
             if self.data.get('receptor_residencia_fiscal') != 'MEX':
                 receptor.setAttribute('ResidenciaFiscal', self.data.get('receptor_residencia_fiscal'))
@@ -253,7 +262,11 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
             elif self.complemento == COMPLEMENTO_PAGOS:
                 complemento.appendChild(self.get_complemento_pagos())
             elif self.complemento == COMPLEMENTO_CARTA_PORTE:
-                complemento.appendChild(self.get_complemento_cartaporte())
+                cartaporte_data = self.data.get('complemento_cartaporte', {})
+                if cartaporte_data and cartaporte_data.get('Version') == '3.0':
+                    complemento.appendChild(self.get_complemento_cartaporte30())
+                else:
+                    complemento.appendChild(self.get_complemento_cartaporte20())
             return complemento
         except Exception as e:
             raise Exception("Ocurrio un error al agregar los complementos. Error::: " + str(e))
@@ -550,7 +563,7 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
         except Exception as e:
             raise Exception("Ocurrio un error. Error::: " + str(e))
 
-    def get_complemento_cartaporte(self):
+    def get_complemento_cartaporte20(self):
         try:
             cartaporte = self.dom.createElement('cartaporte20:CartaPorte')
             cartaporte.setAttribute('Version', '2.0')
@@ -616,9 +629,6 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
                 cartaporte_mercancia.setAttribute('ValorMercancia', mercancia_data.get('ValorMercancia'))
                 cartaporte_mercancia.setAttribute('Moneda', mercancia_data.get('Moneda'))
 
-                if mercancia_data.get('MaterialPeligroso') == 'No':
-                    cartaporte_mercancia.setAttribute('MaterialPeligroso', mercancia_data.get('MaterialPeligroso', ''))
-
                 if mercancia_data.get('MaterialPeligroso') == 'Sí':
                     cartaporte_mercancia.setAttribute('MaterialPeligroso', mercancia_data.get('MaterialPeligroso'))
                     cartaporte_mercancia.setAttribute('CveMaterialPeligroso', mercancia_data.get('CveMaterialPeligroso'))
@@ -684,6 +694,157 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
 
             cartaporte.appendChild(cartaporte_figura_transporte)
 
+            return cartaporte
+        except Exception as e:
+            raise Exception("Ocurrio un error al generar el complemento cartaporte. Error::: " + str(e))
+
+    def get_complemento_cartaporte30(self):
+        try:
+            cartaporte_data = self.data.get('complemento_cartaporte')
+            cartaporte = self.dom.createElement('cartaporte30:CartaPorte')
+            cartaporte.setAttribute('Version', '3.0')
+            cartaporte.setAttribute('IdCCP', cartaporte_data.get('IdCCP')) # nuevo - identificador complemento carta porte
+            cartaporte.setAttribute('TranspInternac', cartaporte_data.get('TranspInternac'))
+            if cartaporte_data.get('TranspInternac') == 'Si':
+                if cartaporte_data.get('EntradaSalidaMerc', '') not in ['Entrada', 'Salida']:
+                    raise Exception('El valor del campo EntradaSalidaMerc debe ser: Entrada ó Salida')
+                cartaporte.setAttribute('RegimenAduanero', cartaporte_data.get('RegimenAduanero'))  # nuevo
+                cartaporte.setAttribute('EntradaSalidaMerc', cartaporte_data.get('EntradaSalidaMerc'))
+                cartaporte.setAttribute('PaisOrigenDestino', cartaporte_data.get('PaisOrigenDestino'))
+                cartaporte.setAttribute('ViaEntradaSalida', cartaporte_data.get('ViaEntradaSalida'))
+
+            cartaporte_ubicaciones = self.dom.createElement('cartaporte30:Ubicaciones')
+            ubicaciones = cartaporte_data.get('ubicaciones', [])
+            total_distancia_recorrida = 0
+            for ubicacion_data in ubicaciones:
+                cartaporte_ubicacion = self.dom.createElement('cartaporte30:Ubicacion')
+                destination_counter = 1
+                if ubicacion_data.get('type') == 'origen':
+                    cartaporte_ubicacion.setAttribute('IDUbicacion', 'OR{}'.format('000001'))
+                    cartaporte_ubicacion.setAttribute('TipoUbicacion', 'Origen')
+
+                elif ubicacion_data.get('type') == 'destino':
+                    cartaporte_ubicacion.setAttribute('IDUbicacion', 'DE00000{}'.format(destination_counter))
+                    cartaporte_ubicacion.setAttribute('TipoUbicacion', 'Destino')
+
+                cartaporte_ubicacion.setAttribute('RFCRemitenteDestinatario', ubicacion_data.get('RFCRemitenteDestinatario'))
+                cartaporte_ubicacion.setAttribute('NombreRemitenteDestinatario', ubicacion_data.get('NombreRemitenteDestinatario'))  # nuevo
+                if ubicacion_data.get('RFCRemitenteDestinatario') == 'XEXX010101000':
+                    cartaporte_ubicacion.setAttribute('NumRegIdTrib', ubicacion_data.get('NumRegIdTrib'))  # nuevo
+                    cartaporte_ubicacion.setAttribute('ResidenciaFiscal', ubicacion_data.get('ResidenciaFiscal'))  # nuevo
+                cartaporte_ubicacion.setAttribute('FechaHoraSalidaLlegada', ubicacion_data.get('FechaHoraSalidaLlegada'))
+
+                if ubicacion_data.get('type') == 'destino':
+                    cartaporte_ubicacion.setAttribute('DistanciaRecorrida', str(ubicacion_data.get('DistanciaRecorrida')))
+                    total_distancia_recorrida += float(ubicacion_data.get('DistanciaRecorrida'))
+                    destination_counter += 1
+
+                cartaporte_ubicacion_domicilio = self.dom.createElement('cartaporte30:Domicilio')
+                cartaporte_ubicacion_domicilio.setAttribute('Calle', ubicacion_data.get('Calle'))
+                if ubicacion_data.get('NumeroExterior') != '':
+                    cartaporte_ubicacion_domicilio.setAttribute('NumeroExterior', ubicacion_data.get('NumeroExterior'))
+                if ubicacion_data.get('NumeroInterior') != '':
+                    cartaporte_ubicacion_domicilio.setAttribute('NumeroInterior', ubicacion_data.get('NumeroInterior'))
+                if ubicacion_data.get('Colonia') != '':
+                    cartaporte_ubicacion_domicilio.setAttribute('Colonia', ubicacion_data.get('Colonia'))
+                if ubicacion_data.get('Localidad') != '':
+                    cartaporte_ubicacion_domicilio.setAttribute('Localidad', ubicacion_data.get('Localidad'))
+                if ubicacion_data.get('Referencia') != '':
+                    cartaporte_ubicacion_domicilio.setAttribute('Referencia', ubicacion_data.get('Referencia'))
+                if ubicacion_data.get('Municipio') != '':
+                    cartaporte_ubicacion_domicilio.setAttribute('Municipio', ubicacion_data.get('Municipio'))
+                cartaporte_ubicacion_domicilio.setAttribute('Estado', ubicacion_data.get('Estado'))
+                cartaporte_ubicacion_domicilio.setAttribute('Pais', ubicacion_data.get('Pais'))
+                cartaporte_ubicacion_domicilio.setAttribute('CodigoPostal', ubicacion_data.get('CodigoPostal'))
+                cartaporte_ubicacion.appendChild(cartaporte_ubicacion_domicilio)
+                cartaporte_ubicaciones.appendChild(cartaporte_ubicacion)
+            cartaporte.appendChild(cartaporte_ubicaciones)
+
+            if cartaporte_data.get('tipo_transporte') in ['AutotransporteFederal', 'TransporteFerroviario']:
+                cartaporte.setAttribute('TotalDistRec', str(round(total_distancia_recorrida, 3)))
+
+            cartaporte_mercancias = self.dom.createElement('cartaporte30:Mercancias')
+            mercancias = cartaporte_data.get('mercancias', [])
+            num_total_mercancias = 0
+            peso_bruto = 0
+            for mercancia_data in mercancias:
+                cartaporte_mercancia = self.dom.createElement('cartaporte30:Mercancia')
+                if mercancia_data.get('BienesTransp'):
+                    cartaporte_mercancia.setAttribute('BienesTransp', mercancia_data.get('BienesTransp'))
+                    cartaporte_mercancia.setAttribute('Descripcion', mercancia_data.get('Descripcion'))
+                    cartaporte_mercancia.setAttribute('Cantidad', mercancia_data.get('Cantidad'))
+                    cartaporte_mercancia.setAttribute('ClaveUnidad', mercancia_data.get('ClaveUnidad'))
+                cartaporte_mercancia.setAttribute('PesoEnKg', mercancia_data.get('PesoEnKg'))
+                cartaporte_mercancia.setAttribute('ValorMercancia', mercancia_data.get('ValorMercancia'))
+                cartaporte_mercancia.setAttribute('Moneda', mercancia_data.get('Moneda'))
+
+                if mercancia_data.get('MaterialPeligroso') == 'Sí':
+                    cartaporte_mercancia.setAttribute('MaterialPeligroso', mercancia_data.get('MaterialPeligroso', ''))
+                    cartaporte_mercancia.setAttribute('CveMaterialPeligroso', mercancia_data.get('CveMaterialPeligroso'))
+                    cartaporte_mercancia.setAttribute('Embalaje', mercancia_data.get('Embalaje'))
+                    cartaporte_mercancia.setAttribute('DescripEmbalaje', mercancia_data.get('DescripEmbalaje'))
+
+                cartaporte_mercancias.appendChild(cartaporte_mercancia)
+                num_total_mercancias += 1
+                peso_bruto += float(mercancia_data.get('PesoEnKg'))
+
+                cartaporte_mercancia_cantidad_transporta = self.dom.createElement('cartaporte30:CantidadTransporta')
+                cartaporte_mercancia_cantidad_transporta.setAttribute('Cantidad', mercancia_data.get('Cantidad'))
+                cartaporte_mercancia_cantidad_transporta.setAttribute('IDOrigen', 'OR000001')
+                cartaporte_mercancia_cantidad_transporta.setAttribute('IDDestino', 'DE000001')
+                cartaporte_mercancia.appendChild(cartaporte_mercancia_cantidad_transporta)
+
+            cartaporte_mercancias.setAttribute('NumTotalMercancias', str(num_total_mercancias))
+            cartaporte_mercancias.setAttribute('PesoBrutoTotal', str(round(peso_bruto, 3)))
+            cartaporte_mercancias.setAttribute('UnidadPeso', 'KGM')
+            cartaporte.appendChild(cartaporte_mercancias)
+
+            if cartaporte_data.get('tipo_transporte') not in ['AutotransporteFederal', 'TransporteMaritimo', 'TransporteAereo', 'TransporteFerroviario']:
+                raise Exception('Se debe registrar un tipo de transporte')
+            transporte_data = cartaporte_data.get('transporte')
+            if cartaporte_data.get('tipo_transporte') == 'AutotransporteFederal':
+                cartaporte_tipo_transporte = self.dom.createElement('cartaporte30:Autotransporte')
+                cartaporte_tipo_transporte.setAttribute('PermSCT', transporte_data.get('PermSCT'))
+                cartaporte_tipo_transporte.setAttribute('NumPermisoSCT', transporte_data.get('NumPermisoSCT'))
+
+                cartaporte_identificacion_vehicular = self.dom.createElement('cartaporte30:IdentificacionVehicular')
+                cartaporte_identificacion_vehicular.setAttribute('ConfigVehicular', transporte_data.get('ConfigVehicular'))
+                cartaporte_identificacion_vehicular.setAttribute('PesoBrutoVehicular', transporte_data.get('PesoBrutoVehicular'))
+                cartaporte_identificacion_vehicular.setAttribute('PlacaVM', transporte_data.get('PlacaVM'))
+                cartaporte_identificacion_vehicular.setAttribute('AnioModeloVM', transporte_data.get('AnioModeloVM'))
+                cartaporte_tipo_transporte.appendChild(cartaporte_identificacion_vehicular)
+
+
+                cartaporte_seguros = self.dom.createElement('cartaporte30:Seguros')
+                cartaporte_seguros.setAttribute('AseguraRespCivil', transporte_data.get('NombreAseg'))
+                cartaporte_seguros.setAttribute('PolizaRespCivil', transporte_data.get('NumPolizaSeguro'))
+
+                cartaporte_tipo_transporte.appendChild(cartaporte_seguros)
+
+                remolques = transporte_data.get('remolques', [])
+                if len(remolques) > 0:
+                    cartaporte_remolques = self.dom.createElement('cartaporte30:Remolques')
+                    for remolque_data in remolques:
+                        cartaporte_remolque = self.dom.createElement('cartaporte30:Remolque')
+                        cartaporte_remolque.setAttribute('SubTipoRem', remolque_data.get('SubTipoRem'))
+                        cartaporte_remolque.setAttribute('Placa', remolque_data.get('Placa'))
+                        cartaporte_remolques.appendChild(cartaporte_remolque)
+                    cartaporte_tipo_transporte.appendChild(cartaporte_remolques)
+                cartaporte_mercancias.appendChild(cartaporte_tipo_transporte)
+
+            cartaporte_figura_transporte = self.dom.createElement('cartaporte30:FiguraTransporte')
+
+            operadores = transporte_data.get('operadores', [])
+            for operador_data in operadores:
+                cartaporte_operador = self.dom.createElement('cartaporte30:TiposFigura')
+                cartaporte_operador.setAttribute('TipoFigura', '01')
+                cartaporte_operador.setAttribute('RFCFigura', operador_data.get('RFCFigura'))
+                cartaporte_operador.setAttribute('NombreFigura', operador_data.get('NombreFigura'))
+                cartaporte_operador.setAttribute('NumLicencia', operador_data.get('NumLicencia'))
+                cartaporte_figura_transporte.appendChild(cartaporte_operador)
+
+            cartaporte.appendChild(cartaporte_figura_transporte)
+
             return cartaporte
         except Exception as e:
             raise Exception("Ocurrio un error al generar el complemento cartaporte. Error::: " + str(e))
\ No newline at end of file
