@@ -70,7 +70,7 @@ class AbstractGeneratorXml(object):
         """
         return self.dom.toxml(encoding='UTF-8').decode('utf8')
 
-    def set_certificado(self, certificado64):
+    def set_certificadoMF(self, certificado64):
         """
         Recibe el certificado abierto como archivo
         :param certificado_file:
@@ -91,6 +91,9 @@ class AbstractGeneratorXml(object):
             logging.error('Error al generar el certificado. Error: ' + str(e))
             raise Exception(e)
 
+    def set_certificadoFLO(self, certificado64):
+        self.dom.setAttribute('Certificado', certificado64)
+
     def set_sello(self, certificado_file_pem64):
         """
         Genera el el sello y lo agrega al xml
