@@ -13,4 +13,4 @@ def replace_char(text, de, a):
 
 
 def replace(text):
-    return replace_char(text, 'áéíóúÁÉÍÓÚàèìòùÀÈÌÒÙñÑ', 'aeiouAEIOUaeiouAEIOUnN')
\ No newline at end of file
+    return replace_char(text, 'áéíóúÁÉÍÓÚàèìòùÀÈÌÒÙ', 'aeiouAEIOUaeiouAEIOU')
\ No newline at end of file
